@@ -11,11 +11,8 @@ such as Sun Grid Engine (SGE) or Condor.
 
 =head1 SYNOPSIS
 
- use Grid::Request drm => 'sge';
- my $request = Grid::Request->new();
-
- # Optionally set the job's project, if the configured scheduler requires it.
- $request->project("MyProject");
+ use Grid::Request;
+ my $request = Grid::Request->new( project => "SomeProject" );
 
  $request->times(2);
  $request->command("/path/to/executable");
@@ -44,8 +41,8 @@ such as Sun Grid Engine (SGE) or Condor.
  my $times_wrong_way = $request->get_times(3);
 
  # Finally, submit the request...
- my @job_ids = $request->submit();
- print "The first ID for this request is $job_ids[0].\n";
+ my @id = $request->submit();
+ print "The first ID for this request is $id[0].\n";
 
  # ...and wait for the results. This step is not necessary, only
  # if you wish to block, or wait for the request to complete before
@@ -64,10 +61,6 @@ Grid::Request->new(%args);
 B<Description:> This is the object constructor. Parameters are passed to
 the constructor in the form of a hash. Examples:
 
-  my $req = Grid::Request->new();
-
-  or
-
   my $req = Grid::Request->new( project => "SomeProject" );
 
   or
@@ -81,28 +74,34 @@ the constructor in the form of a hash. Examples:
 Users may also add a "debug" flag to the constructor call for increased
 reporting:
 
-  my $req = Grid::Request->new( debug => 1 );
+  my $req = Grid::Request->new( project => "SomeProject",
+                                debug   => 1 );
 
-B<Parameters:> A hash of request configuration options.
+B<Parameters:> Only the 'project' parameter is mandatory when calling the
+constructor.
 
 B<Returns:> $obj, a Grid::Request object.
 
 =head1 CONFIGURATION
 
-By default, the configuration file that is used to determine what grid engine
-type to use and where to store temporary files is located in the invoking
-user's home directory under ~/.grid_request.conf. The file needs needs to have
-a [request] header and entries for the 'tempdir' and 'drm' parameters.  In addition,
-the file may also specify the path to a Log::Log4perl configuration file with the
-'log4perl-conf' entry name.The following is an example:
+By default, the configuration file to that is used to determine what grid
+engine type to use and where to store temporary files is located in the
+invoking user's home directory under ~/.grid_request.conf. The file needs
+needs to have a [request] header and configurations for the 'tempdir' and
+'drm' parameters. The following is an example:
 
       [request]
       tempdir=/path/to/grid/accessible/tmp/directory
-      log4perl-conf=/path/to/custom-log4perl.conf
+      drm=SGE
 
 The 'tempdir' directory must point to a directory that is accessible
 to the grid execution machines, for instance, over NFS...
+Users may provide an alternate path to a different configuration file
+by specifying the 'config' parameter to the constructor:
 
+  my $req = Grid::Request->new( project => "SomeProject",
+                                config => "/some/other/dir/request.conf",
+                              );
 Another way of specifying an alternate configuration is to define
 the GRID_CONFIG environment variable.
 
@@ -136,9 +135,10 @@ my $section = $Grid::Request::HTC::config_section;
 
 my $WORKER = $Grid::Request::HTC::WORKER;
 
-our $DRM = undef;
+my $command_element = 0;
 my $DRMAA_INITIALIZED = 0;
-our $VERSION = '0.12';
+my $DEFAULT_BLOCK_SIZE = 100;
+our $VERSION = qw$Revision: 8365 $[1];
 my $SESSION_NAME = lc(__PACKAGE__);
 $SESSION_NAME =~ s/:+/_/g;
 
@@ -151,7 +151,7 @@ BEGIN: {
     require 5.006_00; # Make sure we're not running some old Perl.
 
     # Here we set up which methods go where.
-    my @command_meths = qw(account add_param block_size class cmd_type command
+    my @command_meths = qw(account add_param class cmd_type command
                            email end_time getenv project error hosts initialdir
                            input length memory name opsys output
                            priority start_time state times runtime evictable
@@ -163,7 +163,6 @@ BEGIN: {
 }
 
 
-# The constructor.
 sub new {
     my ($class, %args) = @_;
     my $self = bless {}, $class || ref($class);
@@ -193,14 +192,38 @@ sub new {
     return $self;
 }
 
-sub import {
-    my ($module, $drm, $drm_target) = @_;
-    if (defined $drm && $drm eq "drm") {
-        $DRM = $drm_target;
+# Private method only. Used to configure logging.
+sub _init_config_logger {
+    my $config = shift;
+    # TODO: Since Grid::Request::HTC already parsed the default config file, use
+    # an accessor to get that config object rather than reparsing it
+    # here. This will involve adding an additional method in Grid::Request::HTC.
+    my $default_cfg_obj = Config::IniFiles->new( -file => $default_config );
+    my ($cfg, $same_configs);
+    if (defined $config && ($config eq $default_config)) {
+        $same_configs = 1;
+        $cfg = $default_cfg_obj;
+    } else {
+        $cfg = Config::IniFiles->new(-file   => $config,
+                                     -import => $default_cfg_obj);
     }
+
+    # Parse the location of the logger configuration and initialize
+    # Log4perl, if it has not already been initialized.
+    my $logger_conf = $cfg->val($section, "log4perl-conf");
+    Log::Log4perl->init_once($logger_conf);
+
+    # TODO: The currently installed version of Log::Log4perl (0.34) exhibits
+    # a problem when you check the return value of init. Per the perldoc
+    # documentation, you should be able to do an "or die..." on the init
+    # call, but it fails every time then. Check later releases, and if it's
+    # fixed, install and uncomment the next line.
+    #    or croak "Could not initialize logging with $logger_conf.";
+    $logger = get_logger(__PACKAGE__);
+
+    return $cfg;
 }
 
-# Initialize the object. This is a private method. Do not call directly.
 sub _init {
     # Initialize the Request object. We need to parse the configuration
     # file, initialize the logger, create the Command object.
@@ -213,18 +236,14 @@ sub _init {
     # temporary files and other information that user configuration files do
     # not need (or should) know about.
 
-    my $cfg = _get_config($config);
-    _init_logger($cfg);
+    my $cfg = _init_config_logger($config);
 
     $logger->info("Creating the first Command object.");
     # holds an array of command elements.
-    $self->{_cmd_ele}->[0] =
+    $self->{_cmd_ele}->[$command_element] =
         Grid::Request::Command->new(%$command_args_ref);
 
-    if (! defined $DRM) {
-        $DRM = _get_drm_from_config($cfg);
-    }
-    $self->{_drm} = _load_drm($DRM);
+    $self->{_drm} = _load_drm($cfg);
     $self->{_simulate} = 0;   # For simulate.
     $self->{_submitted} = 0;  # For the submitted flag.
     $self->{_config} = $cfg;  # For the configuration object.
@@ -233,66 +252,19 @@ sub _init {
     _init_drmaa();
 }
 
-# Private method only. Used to configure logging.
-sub _get_config {
-    my $config = shift;
-
-    # TODO: Since Grid::Request::HTC already parsed the default config file, use
-    # an accessor to get that config object rather than reparsing it
-    # here. This will involve adding an additional method in Grid::Request::HTC.
-    my $default_cfg_obj = Config::IniFiles->new( -file => $default_config );
-    my $cfg;
-    if (defined $config && ($config eq $default_config)) {
-        # The configs are the same
-        $cfg = $default_cfg_obj;
-    } else {
-        $cfg = Config::IniFiles->new(-file   => $config,
-                                     -import => $default_cfg_obj);
-    }
-
-    return $cfg;
-}
-
-sub _init_logger {
-    my $cfg = shift;
-
-    # Parse the location of the logger configuration and initialize
-    # Log4perl, if it has not already been initialized.
-    my $logger_conf = $cfg->val($section, "log4perl-conf");
-    if (defined $logger_conf) {
-        if (-f $logger_conf && -r $logger_conf) {
-            Log::Log4perl->init_once($logger_conf);
-        } else {
-            warn "Unable to configure logging with $logger_conf.\n";
-        }
-    }
-
-    $logger = get_logger(__PACKAGE__);
-}
-
 # Accessors (private) used to return internal objects.
 sub _drm { return $_[0]->{_drm}; }
-sub _com_obj_list { return wantarray ? @{ $_[0]->{_cmd_ele} } : $_[0]->{_cmd_ele}; }
-sub _com_obj { return $_[0]->{_cmd_ele}->[-1]; }
+sub _com_obj_list { return $_[0]->{_cmd_ele}; }
+sub _com_obj { return $_[0]->{_cmd_ele}->[$command_element]; }
 sub _config { return $_[0]->{_config}; }
 
-sub _get_drm_from_config {
-    my $cfg = shift;
-    my $param = "drm";
-    my $drm = uc($cfg->val($section, $param));
-
-    if (! defined $drm || length($drm) == 0) {
-        Grid::Request::Exception->throw("No $param parameter specified in the config file " . $cfg->GetFileName());
-    }
-
-    return $drm;
-}
-
 sub _load_drm {
     $logger->debug("In _load_drm.");
-    my $drm = uc(shift);
+    my $cfg = shift;
+    my $param = $Grid::Request::HTC::drm_param;
+    my $drm = uc($cfg->val($section, $param));
     if (! defined $drm) {
-        Grid::Request::Exception->throw("No $drm specified!");
+        Grid::Request::Exception->throw("No $param parameter specified in the config file " . $cfg->GetFileName());
     }
     my $package = __PACKAGE__ . "::DRM::$drm";
     my $load_result = eval "require $package";
@@ -342,12 +314,12 @@ sub _validate {
     my $self = shift;
     $logger->debug("In _validate.");
     my $rv = 1;
-    if ($self->project() && ($self->project() =~ m/\s/)) {
-        Grid::Request::Exception->throw("White space is not allowed for the 'project' attribute.");
+    if ($self->project() =~ m/\s/) {
+        Grid::Request::Exception->throw("White space is not allowed for the project attribute.");
     }
 
-    if (defined($self->account()) && length($self->account()) && $self->account() =~ m/\s/) {
-        Grid::Request::Exception->throw("White space is not allowed for the 'account' attribute.");
+    if ($self->account() =~ m/\s/) {
+        Grid::Request::Exception->throw("White space is not allowed for the account attribute.");
     }
 
     $logger->debug("Returning $rv.");
@@ -355,6 +327,7 @@ sub _validate {
 }
 
 
+
 # This method knows how to dispatch method invocations to the proper module
 # or class by checking the name against the hashes set up early in this
 # module. The hashes are used to look up which methods go where.
@@ -389,16 +362,14 @@ sub AUTOLOAD {
 # method. Don't modify or remove unless you know what you are doing.
 # This is a private method that is not to be invoked directly.
 sub DESTROY { 
-    # Close the DRMAA Session (if necessary).
-    if ( $DRMAA_INITIALIZED != 0 ) {
-        $logger->debug("Closing the DRMAA session.");
-        my ($error, $diagnosis) = drmaa_exit();
-        if ($error) {
-            $logger->error("Error closing the DRMAA session: ",
-                     drmaa_strerror($error), $diagnosis);
-        } else {
-            $DRMAA_INITIALIZED = 0;
-        }
+    # Close the DRMAA Session
+    $logger->debug("Closing the DRMAA session.");
+    my ($error, $diagnosis) = drmaa_exit();
+    if ($error) {
+        $logger->error("Error closing the DRMAA session: ",
+                 drmaa_strerror($error), $diagnosis);
+    } else {
+        $DRMAA_INITIALIZED = 0;
     }
 }
 
@@ -414,128 +385,6 @@ set (or reset) the account attribute for the command.
 
 B<Returns:> The currently set account (if called with no parameters).
 
-=item $obj->add_array_param($argument, $array_ref);
-
-B<Description:> Add an iterable command line argument to the executable when it
-is run on the grid. This is a higher level method that wraps add_param for
-easier iteration over elements in an array. The following two method calls are
-equivalent:
-
-    $request->add_array_param('--arg=$(Name)', \@values);
-
-    $request->add_param({ type   => "ARRAY",
-                          key    => '--arg=$(Name)',
-                          value  => \@values,
-                        });
-
-B<Parameters:> The first parameter is the the template for the argument for the
-command's execution. The second argument is the directory to iterate over. The
-name of each file in the directory will be token replaced into occurrences of
-'$(Name)' in the template (first argument).
-
-B<Returns:> None.
-
-=cut
-
-sub add_array_param {
-    my ($self, $arg_template, $array_ref) = @_;
-    $logger->debug("In add_array_param.");
-
-    if (! defined($arg_template) || ! defined($array_ref)) {
-        Grid::Request::InvalidArgumentException->throw("add_array_param() requires 2 arguments.");
-    }
-
-    if (ref($array_ref) ne "ARRAY" || length(@$array_ref) == 0) {
-        Grid::Request::InvalidArgumentException->throw("Empty or invalid array reference provided.");
-    }
-
-    $self->add_param({ type  => "ARRAY",
-                       key   => $arg_template,
-                       value => $array_ref });
-                      
-}
-
-
-=item $obj->add_dir_param($argument, $path_to_dir);
-
-B<Description:> Add an iterable command line argument to the executable when it is
-run on the grid. This is a higher level method that wraps add_param for easier
-iteration over files in a directory. The following two method calls are equivalent:
-
-    $request->add_dir_param('--arg=$(Name)', "/path/to/some/directory");
-
-    $request->add_param({ type   => "DIR",
-                          key    => '--arg=$(Name)',
-                          value  => "/path/to/some/directory",
-                        });
-
-
-
-B<Parameters:> The first parameter is the the template for the argument for the
-command's execution. The second argument is the directory to iterate over. The
-name of each file in the directory will be token replaced into occurrences of
-'$(Name)' in the template (first argument).
-
-B<Returns:> None.
-
-=cut
-
-sub add_dir_param {
-    my ($self, $arg_template, $directory) = @_;
-    $logger->debug("In add_dir_param.");
-
-    if (! defined($arg_template) || ! defined($directory)) {
-        Grid::Request::InvalidArgumentException->throw("add_dir_param() requires 2 arguments.");
-    }
-
-    if (length($directory) == 0 || ! -d $directory) {
-        Grid::Request::InvalidArgumentException->throw("Invalid or non-existent directory ($directory).");
-    }
-
-    $self->add_param({ type  => "DIR",
-                       key   => $arg_template,
-                       value => $directory });
-                      
-}
-
-=item $obj->add_file_param($argument, $path_to_file);
-
-B<Description:> Add an iterable command line argument to the executable when it is
-run on the grid. This is a higher level method that wraps add_param for easier
-iteration over lines in a file. The following two method calls are equivalent:
-
-    $request->add_file_param('--arg=$(Name)', "/path/to/some/file");
-
-    $request->add_param({ type   => "FILE",
-                          key    => '--arg=$(Name)',
-                          value  => "/path/to/some/file",
-                        });
-
-B<Parameters:> The first parameter is the the template for the argument for the
-command's execution. The second argument is the file to iterate over. The text
-in each line inside the file will be token replaced into occurrences of
-'$(Name)' in the template (first argument).
-
-B<Returns:> None.
-
-=cut
-
-sub add_file_param {
-    my ($self, $arg_template, $file) = @_;
-    $logger->debug("In add_file_param.");
-
-    if (! defined($arg_template) || ! defined($file)) {
-        Grid::Request::InvalidArgumentException->throw("add_file_param() requires 2 arguments.");
-    }
-
-    if (length($file) == 0 || ! -f $file) {
-        Grid::Request::InvalidArgumentException->throw("Invalid or non-existent file ($file).");
-    }
-
-    $self->add_param({ type  => "FILE",
-                       key   => $arg_template,
-                       value => $file });
-}
 
 =item $obj->add_param($scalar | @list | %hash );
 
@@ -551,8 +400,7 @@ argument, no logic is attempted to interpret the string provided. The module
 simply adds the specified string verbatim to the list of parameters when
 building the command line to invoke on the grid.  If 3 parameters are passed,
 then they are read as "key", "value", "type". The parameter 'type' can be
-either "ARRAY", "DIR", "PARAM", or "FILE" (the default is "PARAM"). Any other
-number of parameters passed will yield an error.
+either "ARRAY", "DIR", "PARAM", or "FILE" (the default is "PARAM").
 
 The 'type' is used in the following way to aid in the parallelization of
 processes: If ARRAY is used, the job will be iterated over the elements of the
@@ -655,48 +503,6 @@ sub add_param {
     return $return;
 }
 
-=item $obj->block_size( [ $scalar | $code_ref ] );
-
-B<Description:> By default, Master/Worker (mw) jobs have a default block size
-of 100.  That is to say, that each worker on the grid will process 100 elements
-of the overall pool of job invocations. However, this isn't always appropriate.
-The user may override the default block size by calling this method and setting
-the block size to an alternate value (a positive integer). The user may also
-provide an anonoymous subroutine (code reference) so that the block size can be
-computed dynamically. If choosing to pass a subroutine , the code reference
-will be passed two arguments: the Grid::Request::Command object that will be
-invoked, and the number of elements that will be iterated over, in that order.
-The subroutine can then use these pieces of information to compute the block
-size. The subroutine MUST return a positive integer scalar or an exception will
-be thrown.
-
-    Examples:
-        # simple scalar block size
-        $request->block_size(1000);
-
-        # Passing a code ref, to make the block size dependent on the
-        # executable...
-        $request->block_size(
-                      sub {
-                          my $com_obj = shift;
-                          my $count = shift;
-
-                          my $exe = $com_obj->command();
-
-                          my $block_size = 50;
-                          if ($exe =~ m/sort/i) {
-                              $block_size = ($count > 100000) ? 10000 : 1000;
-                          }
-                          return $block_size;
-                      }
-                  );
-
-B<Parameters:> A positive integer scalar, or an anonymous subroutine/code
-reference.
-
-B<Returns:> The block size scalar or code reference if called as an accessor
-(no-arguments). If the block size has not been explicitly set, then the default
-block size is returned. No return if called as a mutator.
 
 =item $obj->class([$class]);
 
@@ -847,11 +653,12 @@ sub is_submitted {
 }
 =item $obj->project([$project]);
 
-B<Description:> The project attribute is used to affiliate usage of the DRM
-with a particular administrative project. This will allow for more effective
+B<Description:> The project attribute is used to affiliate usage of the DRM with
+a particular administrative project. This will allow for more effective
 control and allocation of resources, especially when high priority projects
-must be fulfilled. The caller may change the project setting as long as the job
-has not yet been submitted (after submission most request attributes are
+must be fulfilled. Therefore, the "project" is mandatory when the request object
+is built. However, the user may still change the project attribute as long as
+the job has not yet been submitted (after submission most attributes are
 locked).
 
 B<Parameters:> The first parameter will be used to set (or reset)
@@ -888,6 +695,18 @@ B<Returns:> When called with no arguments, returns the currently set
 initialdir, or undef if not yet set.
 
 
+=item $obj->length([length]);
+
+B<Description:> This method is used to characterize how long the request
+is expected to take to complete. For long running requests, an attempt to
+match appropriate resources is made. If unsure, leave this setting unset.
+
+B<Parameters:> "short", "medium", "long". No attempt is made to validate
+the length passed in when used as a setter.
+
+B<Returns:> The currently set length attribute (when called with no
+arguments).
+
 =item $obj->name([name]);
 
 B<Description:> The name attribute for request objects is optional and is
@@ -899,6 +718,19 @@ B<Returns:> When called with no arguments, returns the current name, or
 undef if not yet set. The name cannot be changed once a request is submitted.
 
 
+=item $obj->next_command();
+
+B<Description:> The module allows for requests to encapsulate multiple
+commands. This method will start work on a a new one by moving a cursor.
+Commands are processed in the order in which they are created. In addition,
+the only attribute that the new command inherits from the command that
+preceded it, is the project. However, users are free to change the project by
+calling the project() method...
+
+B<Parameters:> None.
+
+B<Returns:> None.
+
 =item $obj->opsys([$os]);
 
 B<Description:> The default operating system that the request will be processed
@@ -936,52 +768,10 @@ B<Returns:> When called with no arguments, returns the hosts if set.
 
 B<Description:> Used to set the minimum amount of physical memory needed.
 
-B<Parameters:> memory in megabytes. Examples: 1000MB, 5000MB
+B<Parameters:> memory in megabytes, example 10MB, 512MB
 
 B<Returns:> When called with no arguments, returns the memory if set.
 
-=item $obj->new_command();
-
-B<Description:> The module allows for requests to encapsulate multiple
-commands. This method will start work on a a new one by moving a cursor.
-Commands are processed in the order in which they are created if they are
-submitted synchronously, or in parallel if submitted asynchronously (the
-default). In addition, the only attribute that the new command inherits from
-the command that preceded it, is the project (if set). However, users are free
-to change the project by calling the project() method...
-
-B<Parameters:> None.
-
-B<Returns:> None.
-
-=cut
-
-sub new_command {
-    $logger->debug("In new_command.");
-    
-    my ($self, @args) = @_;
-    if (scalar @args) {
-        Grid::Request::InvalidArgumentException->throw("No arguments are valid for new_command().");
-    }
-
-    # The only piece of information replicated from command to command is the
-    # project. So we first get the project and then use it to build the new
-    # Command object.
-    my $project = $self->project();
-
-    # Increment element pointer.
-    my @objs = $self->_com_obj_list;
-    my $cmd_len = scalar(@objs);
-
-    $logger->debug("Creating new Command object in element $cmd_len.");
-    if (defined $project && length($project)) {
-        $self->_com_obj_list->[$cmd_len] =
-            Grid::Request::Command->new( project => $project );
-    } else {
-        $self->_com_obj_list->[$cmd_len] = Grid::Request::Command->new();
-    }
-}
-
 
 =item $obj->pass_through([pass_value]);
 
@@ -991,10 +781,27 @@ request's requirements. Such pass throughs are forwarded unchanged. This is an
 advanced option and should only be used by those familiar with the the
 underlying DRM.
 
-B<Parameters:> $string, a scalar.
+B<Parameters:> $string
 
 B<Returns:> None.
 
+=cut
+
+sub new_command {
+    $logger->debug("In new_command.");
+    my $self = shift;
+
+    # The only piece of information replicated from command to command is the
+    # project. So we first get the project and then use it to build the new
+    # Command object.
+    my $project = $self->project();
+
+    # Increment element pointer.
+    $command_element++;
+    $logger->debug("Creating Command object in element $command_element.");
+    $self->_com_obj_list->[$command_element] =
+        Grid::Request::Command->new( project => $project );
+}
 
 =item $obj->output([path]);
 
@@ -1038,20 +845,11 @@ B<Parameters:> Scalar priority value.
 
 B<Returns:> The current priority, or undef if unset.
 
-=item $obj->runtime([minutes]);
-
-B<Description:> Cap the runtime of the job on the DRM. Most DRM systems
-have a mechanism to limit the maximum amount of time a job can run. This
-method accepts a value in minutes.
-
-B<Parameters:> Scalar containing a positive integer nubmer of minutes.
-
-B<Returns:> The current runtime limit if called as a getter, or undef if unset.
-
+=cut
 
 =item $obj->set_env_list(@vars);
 
-B<Description:> This method is used to establish the environment that a
+B<Description:> This method is used to establish the environment that a a
 request to the grid should run under. Users may pass this method a list of
 strings that are in "key=value" format. The keys will be converted into
 environment variables set to "value" before execution of the command is begun.
@@ -1079,7 +877,7 @@ sub set_env_list {
         push(@valid, $arg);
     }
 
-    $self->{_env} = \@valid;
+    $self->[5] = \@valid;
 
     # If the user has set their own environment with set_envlist, then we
     # assume that they want getenv to be true. We do it for them here to save
@@ -1088,63 +886,6 @@ sub set_env_list {
 }
 
 
-=item $obj->show_invocations();
-
-B<Description:> Show what will be executed on the DRM. An attempt is made
-to excape shell sensitive characters so that the commands can be copied
-and pasted for test execution.
-
-B<Parameters:> None.
-
-B<Returns:> None. The method print to STDOUT.
-
-=cut
-
-sub show_invocations {
-    my $self = shift;
-
-    # Remember, we may have multiple command objects...
-    my @command_objs = $self->_com_obj_list();
-
-    # Iterate over them...
-    my $command_count = 1;
-
-    eval { require Grid::Request::JobFormulator };
-    my $formulator = Grid::Request::JobFormulator->new();
-
-    foreach my $com_obj (@command_objs) {
-        print "Command #" . $command_count . "\n"; 
-        my $exe = $com_obj->command();
-        my $block_size = $com_obj->block_size();
-        my @params = $com_obj->params();
-        my @param_strings = ();
-
-        foreach my $param_obj (@params) {
-            my $param_str = $param_obj->to_string();
-            push (@param_strings, $param_str);
-        }
-
-        my @invocations = $formulator->formulate($block_size, $exe, @param_strings);
-        foreach my $invocations (@invocations) {
-            my @cli = @$invocations;
-            my @esc_cli = _esc_chars(@cli);
-            print join(" ", @esc_cli) . "\n";
-        }
-        
-        $command_count++;
-    }
-}
-
-sub _esc_chars {
-    # will change, for example, a!!a to a\!\!a
-    my @cli = @_;
-    my $e;
-    @cli = map { $e =$_; $e =~ s/([;<>\*\|`&\$!#\(\)\[\]\{\}:'"])/\\$1/g; $e } @cli;
-    @cli = map { if ($_ =~ m/\s/) { '"' . $_ . '"' } else { $_ } } @cli;
-    return @cli;
-}
-
-
 =item $obj->simulate([value]);
 
 B<Description:> This method is used to toggle the simulate flag for the
@@ -1316,9 +1057,7 @@ sub _throw_drmaa {
 # implementation.
 sub _init_drmaa {
     $logger->debug("In _init_drmaa.");
-    if ($DRMAA_INITIALIZED) {
-        $logger->debug("DRMAA session aready initialized.");
-    } else {
+    unless ($DRMAA_INITIALIZED) {
         my ($error, $diagnosis) = drmaa_init("session=$SESSION_NAME");
         _throw_drmaa("Could not initialize DRMAA", $error, $diagnosis) if $error;
         $DRMAA_INITIALIZED = 1;
@@ -1354,41 +1093,23 @@ sub _drmaa_submit {
     my @ids;
     my $total = $self->command_count();
     my $count = 1;
-
-    eval {
-        for (my $cmd = 0; $cmd < $total; $cmd++) {
-            my $cmd_obj = $self->_com_obj_list->[$cmd];
-            $logger->debug("Submitting command $count/$total.");
-            my $jt = $self->_cmd_base_drmaa($cmd_obj);
-            $logger->debug("Got a good job template.") if defined $jt;
-            my @sub_ids;
-            if ($cmd_obj->cmd_type() eq "mw") {
-                @sub_ids = $self->_submit_mw($jt, $cmd_obj);
-            } else {
-                @sub_ids = $self->_submit_htc($jt, $cmd_obj);
-            }
-            if ($serially) {
-                _sync_ids($cmd_obj);
-            }
-            push @ids, @sub_ids;
-            $count++;
+    for (my $cmd = 0; $cmd < $total; $cmd++) {
+        my $cmd_obj = $self->_com_obj_list->[$cmd];
+        $logger->debug("Submitting command $count/$total.");
+        my $jt = $self->_cmd_base_drmaa($cmd_obj);
+        $logger->debug("Got a good job template.") if defined $jt;
+        my @sub_ids;
+        if ($cmd_obj->cmd_type() eq "mw") {
+            @sub_ids = $self->_submit_mw($jt, $cmd_obj);
+        } else {
+            @sub_ids = $self->_submit_htc($jt, $cmd_obj);
         }
-    };
-
-    my $e;
-    if ($e = Exception::Class->caught("Grid::Request::DRMAAException")) {
-        $logger->fatal("Unable to submit: " . $e->diagnosis() . ": " . $e->drmaa() );
-        $e->rethrow();
-    } elsif ( $e = Exception::Class->caught("Grid::Request::Exception") ) {
-        $logger->fatal("Unable to submit: " . $e->error());
-        $e->rethrow();
-    } else {
-        $e = Exception::Class->caught();
-        if ($e) {
-            ref $e ? $e->rethrow : $logger->logcroak("Unable to submit: $e");
+        if ($serially) {
+            _sync_ids($cmd_obj);
         }
+        push @ids, @sub_ids;
+        $count++;
     }
-
     $logger->debug("Finished submitting.");
     return wantarray ? @ids : \@ids;
 }
@@ -1415,8 +1136,8 @@ sub _cmd_base_drmaa {
     my $output = $cmd->output();
     my $error_path = $cmd->error();
     if (defined($output) || defined($error_path)) {
-        $output =~ s/\$\(Index\)/\$drmaa_incr_ph\$/g if defined($output);
-        $error_path =~ s/\$\(Index\)/\$drmaa_incr_ph\$/g if defined($error_path);
+        $output =~ s/\$\(Index\)/\$drmaa_incr_ph\$/g;
+        $error_path =~ s/\$\(Index\)/\$drmaa_incr_ph\$/g;
 
         $output ||= '/dev/null';
         $logger->debug("STDOUT will go to $output.");
@@ -1459,7 +1180,7 @@ sub _cmd_base_drmaa {
     my $email = $cmd->email();
     if ($email) {
         $logger->info("Setting DRM to not block emails.");
-        ($error, $diagnosis) = drmaa_set_attribute($jt, $DRMAA_BLOCK_EMAIL, "0");
+        ($error, $diagnosis) = drmaa_set_attribute($jt, $DRMAA_BLOCK_EMAIL, 0);
         _throw_drmaa("Unable to unblock emails.", $error, $diagnosis) if $error;
         $logger->info("Setting the job email.");
         ($error, $diagnosis) = drmaa_set_vector_attribute($jt, $DRMAA_V_EMAIL, [$email]);
@@ -1467,7 +1188,7 @@ sub _cmd_base_drmaa {
     }
  
     my @drm_methods = qw(account hosts opsys evictable priority memory
-                         project class runtime);
+                         length project class runtime);
     my @native_attrs;
     foreach my $method (@drm_methods) {
         my $val = $cmd->$method;
@@ -1506,7 +1227,7 @@ sub _submit_mw {
     $logger->debug("In _submit_mw.");
     my ($self, $jt, $cmd) = @_;
     unless (defined $jt && defined $cmd) {
-        Grid::Request::InvalidArgumentException->throw("Job template and/or command object are not defined.");
+        Grid::Request::InvalidArgumentException->("Job template and/or command object are not defined.");
     }
 
     $logger->debug("Setting the command executable.");
@@ -1561,31 +1282,23 @@ sub _submit_mw {
     # should be split up differently. This is why we pass $cmd, so that more
     # intelligent analysis may be done if configured...
     
-    my $block_size = $cmd->block_size();
-    if (ref($block_size) eq "CODE") {
-        $logger->debug("Detected a code reference for block size.");
-        my $block_size_calculator = $block_size;
-        $logger->debug("Invoking the code to determine the block size.");
-        $block_size = $block_size_calculator->($cmd, $min_count); 
-
-        if (defined($block_size) && length($block_size) && $block_size =~ /^-?\d+$/) {                                                                          
-            if ($block_size > 0) {                                                                              
-                $logger->debug("Invocation yielded a block size of $block_size.");
-            } else {
-                Grid::Request::Exception->throw(                                                  
-                    "Block size code reference yielded an invalid result. Must be a positive integer.");
-            }
-        } else {
-            Grid::Request::Exception->throw(
-                    "Block size code reference yielded an invalid result.");
-        }
-    } else {
-        $logger->debug("block_size is a regular scalar: $block_size");
-    }
-
-    # Compute the number of workers to invoke based on the block size.
+    my $block_size = $DEFAULT_BLOCK_SIZE;
     my $workers = ceil($min_count / $block_size);
 
+    eval { 
+        my $package = "use Grid::Request::Util";
+        my $load_result = eval "require $package";
+        if (defined($load_result) && ($load_result == 1)) {
+            import $package;
+            # Use some (possibly user defined) technique to deterine the block
+            # size based on what kind of job is being executed on the grid
+            $block_size = Grid::Request::Util->determine_block_size($cmd, $min_count);
+            $workers = ceil($min_count / $block_size);
+        }
+    };
+    if ($@) {
+        $logger->error("Could not load the Grid::Request::Util module. Sticking with the worker count of $workers.");
+    }
     my $plurality = ($workers == 1) ? "worker" : "workers";
     $logger->info("This master/worker command requires $workers $plurality.");
 
@@ -1654,13 +1367,13 @@ sub _submit_htc {
     $logger->debug("In _submit_htc.");
     my ($self, $jt, $cmd) = @_;
     unless (defined $jt && defined $cmd) {
-        Grid::Request::InvalidArgumentException->throw(
+        Grid::Request::InvalidArgumentException->(
             "Job template and/or command object are not defined.");
     }
 
     my $exe = $cmd->command();
     unless (defined $exe) {
-        Grid::Request::InvalidArgumentException->throw("Command executable is not defined.");
+        Grid::Request::InvalidArgumentException->("Command executable is not defined.");
     }
     $logger->debug("Setting the command executable.");
     my ($error, $diagnosis) = drmaa_set_attribute($jt, $DRMAA_REMOTE_COMMAND, $exe);
@@ -1695,14 +1408,9 @@ sub _submit_htc {
         my $job_ids;
         ($error, $job_ids, $diagnosis) = drmaa_run_bulk_jobs($jt, 1, $times, 1);
         _throw_drmaa("Could not run bulk jobs.", $error, $diagnosis) if $error;
-        for (my $time_index = 1; $time_index <= $times; $time_index++) {
+        for (my $i=1; $i<=$times; $i++) {
             my ($error, $job_id) = drmaa_get_next_job_id($job_ids);
-            if ($error) {
-                _throw_drmaa("Problem getting next job id.", $error, $diagnosis);
-                $cmd->state('FAILED');
-            } else {
-                $cmd->state('WAITING');
-            }
+            _throw_drmaa("Problem getting next job id.", $error, $diagnosis) if $error;
             $logger->debug("Adding job id $job_id to the jobs array.");
             push (@ids, $job_id);
         }
@@ -1710,12 +1418,7 @@ sub _submit_htc {
         # If here, this is a singleton type job. Only 1 execution...
         my $job_id;
         ($error, $job_id, $diagnosis) = drmaa_run_job($jt);
-        if ($error) {
-            _throw_drmaa("Error running job.", $error, $diagnosis);
-            $cmd->state('FAILED');
-        } else {
-            $cmd->state('WAITING');
-        }
+        _throw_drmaa("Error running job.", $error, $diagnosis) if $error;
         $logger->debug("Adding job id $job_id to the jobs array.");
         # since the return is for an array of ids, make an array containing
         # a single job id.
@@ -1895,19 +1598,53 @@ sub _sync_ids {
         $logger->debug("Will wait for $wait_time seconds.")
     }
     my @job_ids = $cmd_obj->ids();
-    $cmd_obj->state('RUNNING');
     my ($error, $diagnosis) = drmaa_synchronize(\@job_ids, $wait_time, 0);
-    if ($error) {
-        $cmd_obj->state('FAILED');
-        _throw_drmaa("Error waiting for job execution.", $error, $diagnosis);
+    _throw_drmaa("Error waiting for job execution.", $error, $diagnosis) if $error;
+}
+
+=item $obj->get_tasks();
+
+B<Description:> Retrieve the tasks for this request
+
+B<Parameters:> None.
+
+B<Returns:> A hash of hashes (HoH) representing the tasks for this
+request. The hash is organized by the index and the value
+is another hashref with the actual data. The following is an example
+of the return data structure:
+
+  $hashref = {
+            '1' => {
+                   'returnValue' => 0,
+                   'message'     => undef,
+                   'state'       => 'FINISHED'
+                 },
+            '2' => {
+                   'returnValue' => -1,
+                   'message'     => 'Failed task.',
+                   'state'       => 'FAILED'
+                 }
+          }
+
+=cut
+
+sub get_tasks {
+    $logger->debug("In get_tasks.");
+    my ($self, @args) = @_;
+    my $tasks;
+    if ($self->is_submitted) {
+        # TODO: Implement via DRMAA vice HTC
+    } else {
+        $logger->logwarn("The request must be submitted before get_tasks ",
+                         "may be called.");
     }
-    $cmd_obj->state('FINISHED');
+    return $tasks;
 }
 
 sub get_status {
     my ($class, $job_id) = @_;
     if (! defined $logger) {
-        _init_logger($default_config);
+        _init_config_logger($default_config);
     }
     _init_drmaa();
 
@@ -1920,7 +1657,6 @@ sub get_status {
     my $status;
     my ($error, $remoteps, $diagnosis) = drmaa_job_ps($job_id);
     _throw_drmaa("Could not get status.", $error, $diagnosis) if $error;
-
     if ($remoteps == $DRMAA_PS_RUNNING) {
         $status = "RUNNING";
     } elsif ($remoteps == $DRMAA_PS_QUEUED_ACTIVE) {
